#include "bigscreenbusiness.h"

template <typename T, typename U>
U* allocateAndAssign(const QVector<QVector<QString>>& result, int head_type) {
    int num_rows = result.size();
    size_t size = sizeof(U) + num_rows * sizeof(T);

    U* data = (U*)malloc(size);
    data->head.type = head_type;
    data->head.lenth = size;
    data->count = num_rows;
    for (int i = 0; i < num_rows; ++i) {
       assignValue(data->dataResNum[i], result[i]);
    }
    return data;
}

void assignValue(BS_annual& item, const QVector<QString>& row) {
    QString firstElement = row[0].trimmed();
    QByteArray utf8String = firstElement.toUtf8();
    std::strncpy(item.prod_type, utf8String.constData(), sizeof(item.prod_type) - 1);
    item.prod_type[sizeof(item.prod_type) - 1] = '\0';
    item.prod_num = row[1].toInt();
    item.prod_orders = row[2].toInt();
    item.prod_money = row[3].toInt();
}

void assignValue(BS_material& item, const QVector<QString>& row) {
    if (row.size() >= 4) { // 确保row有足够的元素
           // 将QString转换为C风格的字符串，并复制到char数组
           strncpy(item.material_type, row[0].trimmed().toLatin1().data(), sizeof(item.material_type) - 1);
           item.material_type[sizeof(item.material_type) - 1] = '\0'; // 确保字符串以'\0'结尾

           // 安全地转换QString到int
           bool ok;
           item.maxnum = row[1].toInt(&ok);
           if (!ok) {
               // 处理转换失败的情况，例如设置一个错误代码或打印错误消息
           }

           item.storenum = row[2].toInt(&ok);
           if (!ok) {
               // 处理转换失败的情况
           }

           // 复制状态描述到char数组
           strncpy(item.status, row[3].trimmed().toLatin1().data(), sizeof(item.status) - 1);
           item.status[sizeof(item.status) - 1] = '\0'; // 确保字符串以'\0'结尾
       }
}

void assignValue(BS_month& item, const std::vector<std::string>& row) {
    if (row.size() >= 3) { // 确保row至少有三个元素
        // 将std::string转换为QString，然后转换为char数组
        QString qstrProdType = QString::fromStdString(row[0]);
        std::strncpy(item.prod_type, qstrProdType.toLatin1().constData(), sizeof(item.prod_type) - 1);
        item.prod_type[sizeof(item.prod_type) - 1] = '\0'; // 确保字符串以'\0'结尾

        // 将std::string转换为int
        item.month = std::stoi(row[1]);
        item.num = std::stoi(row[2]);
    }
}

void assignValue(BS_order& item, const QVector<QString>& row) {
    if (row.size() >= 7) { // 确保row有足够的元素
        // 使用QString::toLatin1().data()来获取C字符串，并使用strncpy进行复制
        strncpy(item.order_number, row[0].toLatin1().constData(), sizeof(item.order_number) - 1);
        strncpy(item.product_type, row[1].toLatin1().constData(), sizeof(item.product_type) - 1);
        strncpy(item.start_date, row[2].toLatin1().constData(), sizeof(item.start_date) - 1);
        strncpy(item.end_date, row[3].toLatin1().constData(), sizeof(item.end_date) - 1);
        item.order_number[sizeof(item.order_number) - 1] = '\0';
        item.product_type[sizeof(item.product_type) - 1] = '\0';
        item.start_date[sizeof(item.start_date) - 1] = '\0';
        item.end_date[sizeof(item.end_date) - 1] = '\0';

        // 安全地转换QString到double
        bool ok;
        item.plan_quantity = row[4].toDouble(&ok);
        if (!ok) {
            // 处理转换失败的情况
        }
        item.production_quantity = row[5].toDouble(&ok);
        if (!ok) {
            // 处理转换失败的情况
        }

        strncpy(item.order_status, row[6].toLatin1().constData(), sizeof(item.order_status) - 1);
        item.order_status[sizeof(item.order_status) - 1] = '\0';
    }
}

void assignValue(BS_daily& item, const QVector<QString>& row) {
    if (row.size() >= 2) { // 确保row至少有两个元素
        // 使用strncpy来复制字符串，并确保以'\0'结尾
        QString processStr = row[0].trimmed();
        strncpy(item.process, processStr.toLatin1().constData(), sizeof(item.process) - 1);
        item.process[sizeof(item.process) - 1] = '\0'; // 确保字符串以'\0'结尾

        // 安全地转换QString到int
        bool ok;
        item.num = row[1].toInt(&ok);
        if (!ok) {
            // 处理转换失败的情况，例如设置item.num为0或其他默认值
            item.num = 0;
        }
    }
}

void assignValue(BS_quality& item, const QVector<QString>& row) {
    if (row.size() >= 2) { // 确保row至少有两个元素
        // 使用 strncpy 来复制字符串，并确保以'\0'结尾
        QString qualityTypeStr = row[0].trimmed();
        int maxLength = sizeof(item.quality_type) - 1;
        strncpy(item.quality_type, qualityTypeStr.toLatin1().constData(), maxLength);
        item.quality_type[maxLength] = '\0'; // 确保字符串以'\0'结尾

        // 安全地转换 QString 到 int
        bool ok;
        item.num = row[1].toInt(&ok);
        if (!ok) {
            // 处理转换失败的情况，例如打印错误消息或设置默认值
            qDebug() << "Conversion to int failed for row[1]";
            item.num = 0; // 或者其他适当的默认值
        }
    }
}




BigScreenBusiness::BigScreenBusiness()
{

}

BigScreenBusiness::~BigScreenBusiness()
{

}

int BigScreenBusiness::business(SOCKET client, BS_annualReq *req) {
    (void)client;
    if (req == nullptr) {
        return -1;
    }

    // 构建SQL查询字符串
    QString sql = QString("SELECT pro_type, SUM(plan_quan), COUNT(*), SUM(money) "
                          "FROM bgss_orders WHERE start_date LIKE '%1.%%' "
                          "GROUP BY pro_type")
                  .arg("2024"); // 格式化年份

    qDebug() << "sql:" << sql;

    //MySQLDB mysql;
    QVector<QVector<QString>> result;
//    if (!mysql.execSQL(sql, result)) {
//        // 错误处理
//        qDebug() << "Failed to execute SQL query:" << mysql.lastError().text();
//        return -1;
//    }

    qDebug() << "vector.size:" << result.size();

    // 分配并赋值结果
    auto annual_data = allocateAndAssign<BS_annual, BS_annualRes>(result, BIGSCREEN_ANNUAL_RES);

    // 使用Qt的QTcpSocket发送数据
    QTcpSocket socket;
    socket.connectToHost(QHostAddress::LocalHost, 0); // 连接到本机的某个端口，端口号根据实际情况填写
    if (!socket.waitForConnected(5000)) { // 等待5秒连接
        qDebug() << "Failed to connect to the host.";
        free(annual_data);
        return -1;
    }

    if (socket.write((const char*)annual_data, annual_data->head.lenth) == -1) {
        // 发送失败处理
        qDebug() << "Failed to send data.";
    }

    if (socket.flush()) {
        qDebug() << "annual send ok";
    } else {
        qDebug() << "Failed to flush the data.";
    }

    socket.disconnectFromHost();
    free(annual_data);

    return 0;
}

int BigScreenBusiness::business(SOCKET client, BS_monthReq *req)
{
    if (req == nullptr) {
           return -1;
       }

       // 构建SQL查询字符串
       QString sql = QString("SELECT pro_type, SUBSTRING(start_date, 6, 2) as month, SUM(plan_quan) "
                             "FROM bgss_orders "
                             "WHERE start_date LIKE '%1.%%' "
                             "GROUP BY month, pro_type;")
                     .arg(req->year);
       qDebug() << "sql:" << sql;

       QVector<QVector<QString>> result;
//      if (!mysql.execSQL(sql, result)) {
//          qDebug() << "Failed to execute SQL query:" << mysql.lastError().text();
//          return -1; // 或者设置一个错误代码
//      }

      qDebug() << "Result size:" << result.size();
     auto month_data = allocateAndAssign<BS_month, BS_monthRes>(result, BIGSCREEN_MONTH_RES);
      // 分配并赋值结果
      int sendResult = send(client, reinterpret_cast<const char*>(month_data), month_data->head.lenth, 0);
         if (sendResult == SOCKET_ERROR) {
             qDebug() << "Failed to send data. Error code:" << WSAGetLastError();
         }
         // 释放分配的内存
         free(month_data); // 假设使用了malloc分配
         return 0;
}

int BigScreenBusiness::business(SOCKET client, BS_materialReq *req)
{
    if (req == nullptr) {
           return -1;
       }

       // 查询sql语句，处理结果集
       QString sql = "SELECT material_type, lot_num, total, status "
                     "FROM bgss_raw_materials;";
       qDebug() << "sql:" << sql;

       QSqlQuery query;
       if (!query.exec(sql)) {
           qDebug() << "Failed to execute SQL query:" << query.lastError().text();
           return -1;
       }

       QVector<QVector<QString>> result;
       while (query.next()) {
           QVector<QString> row;
           row.append(query.value(0).toString()); // material_type
           row.append(query.value(1).toString()); // lot_num
           row.append(query.value(2).toString()); // total
           row.append(query.value(3).toString()); // status
           result.append(row);
       }

       // 分配并赋值结果
       BS_materialRes* material_data = allocateAndAssign<BS_material, BS_materialRes>(result, BIGSCREEN_MATERIAL_RES);

         // 使用 SOCKET 的 send 函数发送数据
         int sentBytes = send(client, reinterpret_cast<const char*>(material_data),
                               material_data->head.lenth, 0);
         if (sentBytes == SOCKET_ERROR) {
             qDebug() << "Failed to send data. Error code:" << WSAGetLastError();
         } else {
             qDebug() << "material send ok";
         }
         // 发送完成后释放 material_data 指针
         delete material_data; // 确保使用 delete 释放 new 分配的内存
         return 0;
}

int BigScreenBusiness::business(SOCKET client, BS_orderReq *req)
{
    if (req == nullptr) {
         return -1;
     }

     // 查询sql语句，处理结果集
     QString sql = QString("SELECT order_num, pro_type, start_date, end_date, plan_quan, "
                           "finish_quan, order_status from bgss_orders");
     qDebug() << "sql:" << sql;

     //MySQLDB mysql; // 确保这个类有适当的实现
     QVector<QVector<QString>> result;
     // 执行SQL语句并填充结果（以下代码仅为示例，需要根据实际情况实现）
     // if (!mysql.execSQL(sql, result)) {
     //     qDebug() << "Failed to execute SQL query:" << mysql.lastError().text();
     //     return -1;
     // }

     // 分配并赋值结果
     auto orders_data = allocateAndAssign<BS_order, BS_orderRes>(result, BIGSCREEN_ORDER_RES);

     // 使用send发送数据
     int sentBytes = send(client, reinterpret_cast<char*>(orders_data),
                           orders_data->head.lenth, 0);
     if (sentBytes == SOCKET_ERROR) {
         qDebug() << "Failed to send data. Error code:" << WSAGetLastError();
     } else {
         qDebug() << "order send ok";
     }

     // 此处不需要释放orders_data，因为示例中未显示其分配方式
     // 如果orders_data是new分配的，记得使用delete释放

     return 0;

}

int BigScreenBusiness::business(SOCKET client, BS_dailyReq *req)
{
    if (req == nullptr) {
           return -1;
       }

       // 查询sql语句，处理结果集
       QString sql = QString("SELECT `PROCESS`, COUNT(*) "
                             "FROM bgss_work_in_process "
                             "GROUP BY `process`;");
       qDebug() << "sql:" << sql;

       QSqlQuery query;
       if (!query.exec(sql)) {
           qDebug() << "Failed to execute SQL query:" << query.lastError();
           return -1;
       }

       QVector<QVector<QString>> result;
       while (query.next()) {
           QVector<QString> row;
           row.append(query.value(0).toString()); // PROCESS
           row.append(query.value(1).toString()); // COUNT(*)
           result.append(row);
       }

       // 分配并赋值结果
       auto dailys_data = allocateAndAssign<BS_daily, BS_dailyRes>(result, BIGSCREEN_DAILY_RES);

       if (send(client, reinterpret_cast<char*>(dailys_data), dailys_data->head.lenth, 0) != SOCKET_ERROR) {
           qDebug() << "daily send ok";
       } else {
           qDebug() << "Failed to send data. Error code:" << WSAGetLastError();
       }

       // 发送完成后释放 dailys_data 指针
       // 此处需要根据实际allocateAndAssign函数的实现来决定是否需要释放
       // 如果dailys_data是new分配的，记得使用delete释放

       return 0;
}

int BigScreenBusiness::business(SOCKET client, BS_environmentReq *req)
{
    if (req == nullptr) {
           return -1;
       }

       BS_environmentRes *envRes_data = new BS_environmentRes;
       BS_environment *envir_data = new BS_environment;

       envRes_data->head.type = BIGSCREEN_ENVIRONMENT_RES;
       envRes_data->head.lenth = sizeof(BS_environmentRes);

       qDebug() << req->warning_area
                << " " << req->warning_level
                << " " << req->temperature
                << " " << req->humidity
                << " " << req->particle
                << " " << req->lng
                << " " << req->lat;

       // 复制数据到BS_environment结构体
       strcpy(envir_data->warning_area, req->warning_area);
       strcpy(envir_data->warning_level, req->warning_level);
       envir_data->temperature = req->temperature;
       envir_data->humidity = req->humidity;
       envir_data->particle = req->particle;
       strcpy(envir_data->lng, req->lng);
       strcpy(envir_data->lat, req->lat);

       envRes_data->envir = *envir_data;

       // 使用send发送数据
       if (send(client, reinterpret_cast<char*>(envRes_data), envRes_data->head.lenth, 0) != SOCKET_ERROR) {
           qDebug() << "environment send ok";
       } else {
           qDebug() << "Failed to send data. Error code:" << WSAGetLastError();
       }

       // 释放分配的内存
       delete envRes_data;
       delete envir_data;

       return 0;
}

int BigScreenBusiness::business(SOCKET client, BS_productReq *req)
{
    if (req == nullptr) {
           return -1;
       }

       BS_productRes *product_Res = new BS_productRes;
       BS_product *product_data = new BS_product;

       // 模拟数据
       sprintf(product_data->product_id, "productID%d", rand() % 100);
       sprintf(product_data->status, "status%d", rand() % 3);
       sprintf(product_data->product_time, "productTime%d", rand() % 100);
       product_Res->product = *product_data;

       // 设置响应头信息
       product_Res->head.type = 200; // 假设200是产品响应类型
       product_Res->head.lenth = sizeof(BS_productRes);

       // 使用send发送数据
       if (send(client, reinterpret_cast<char*>(product_Res), product_Res->head.lenth, 0) != SOCKET_ERROR) {
           qDebug() << "product send ok";
       } else {
           qDebug() << "Failed to send product data. Error code:" << WSAGetLastError();
       }

       // 释放分配的内存
       delete product_Res;
       delete product_data;

       return 0;
}

int BigScreenBusiness::business(SOCKET client, BS_qualityReq *req)
{
    if (req == nullptr) {
           return -1;
       }

       // 查询sql语句，处理结果集
       QString sql = QString("SELECT product_status, COUNT(*) "
                             "FROM bgss_finished_product "
                             "GROUP BY product_status;");
       qDebug() << "sql:" << sql;

       QSqlQuery query;
       if (!query.exec(sql)) {
           qDebug() << "Failed to execute SQL query:" << query.lastError();
           return -1;
       }

       QVector<QVector<QString>> result;
       while (query.next()) {
           QVector<QString> row;
           row.append(query.value(0).toString()); // product_status
           row.append(query.value(1).toString()); // COUNT(*)
           result.append(row);
       }

       // 分配并赋值结果
       auto quality_data = allocateAndAssign<BS_quality, BS_qualityRes>(result, BIGSCREEN_QUALITY_RES);

       // 使用send发送数据
       if (send(client, reinterpret_cast<char*>(quality_data), quality_data->head.lenth, 0) != SOCKET_ERROR) {
           qDebug() << "quality data send ok";
       } else {
           qDebug() << "Failed to send quality data. Error code:" << WSAGetLastError();
       }

       // 发送完成后释放 quality_data 指针
       delete quality_data; // 假设使用了new分配

       return 0;
}




